//Contaminated normal-half normal stochastic frontier model
//Coded by Alex Stead, m2i Research Fellow in Efficiency Analysis
//Institute for Transport Studies, University of Leeds
//Email: a.d.stead@leeds.ac.uk
//Version: 30/01/2017

//This programme enables the estimation of a stochastic frontier model in which
//the noise term follows a contaminated normal distribution (i.e. a mixture of
//normals) and the inefficiency term follows a half normal distribution.

program cnsf, eclass
		version 13.1 
		if !replay() {
		syntax varlist(numeric ts) [if] [in] [, Level(cilevel) COST PRODuction]
		local s					=	-1
		if "`cost'"=="" local s	=	1
		global cost_s `s'
		marksample touse
		gettoken yvar xvars : varlist
		ml model d1 cnsf_ll (mu:`yvar'=`xvars') /lnsigma_v1 /lnsigma_v2 /lnsigma_u /logprob, title(Stoch. frontier contaminated normal/half normal model) diparm(lnsigma_v1, exp label(sigma_v1) prob) diparm(lnsigma_v2, exp label(sigma_v2) prob) diparm(lnsigma_u, exp label(sigma_u) prob) diparm(logprob, invlogit label(prob) prob)
		quietly{
		net install sfcross, all from(http://www.econometrics.it/stata)
		sfcross `yvar' `xvars', cost d(h)
		scalar _b_cons=_b[_cons]
		scalar _lnsigma_v1		=	ln(e(sigma_v))+ln(0.5)
		scalar _lnsigma_v2		=	ln(e(sigma_v))+ln(2)
		scalar _lnsigma_u		=	ln(e(sigma_u))
		foreach x in `xvars'{
		scalar _b_`x'			=	_b[`x']
		}
		foreach x in `xvars'{
		ml init mu:`x'			=	`=_b_`x''
		}
		ml init mu:_cons		=	`=_b_cons'
		ml init /lnsigma_v1		=	`=_lnsigma_v1' /lnsigma_v2=`=_lnsigma_v2' /lnsigma_u=`=_lnsigma_u' /logprob=3
		}
		}
		ml search
		ml maximize, difficult
		tempname b V y
		matrix `b'				=	e(b)
		matrix `V'				=	e(V)
		ereturn post `b' `V', esample(`touse')
		ereturn local predict 	"cnsf_p"
		ereturn local cmd		"cnsf"
		global ML_y1 `yvar'
end
